==> ff-global-3.0.0.tgz: package/renderer/src/mqtt.ts <==
//在渲染器进程 (网页) 中。

/*
 * 键：表示已经订阅的主题
 * 值：是一个数组，执行函数
 * */

const topics: {[topic: string]: Function[]} = {}
//当前渲染层id
const webContentId: number = _pre_.getWebContentId()

let userInfo = _pre_.getUserInfo()

//MQTT会用到该文件，但mqtt与gc的全局变量，是在引入后才生成，所以需要延迟调用ff.gc的方法
setTimeout(() => {
  ff.gc.subscribes({
    用户信息渲染进程修改(u) {
      userInfo = Object.assign(userInfo || {}, u)
    }
  })
}, 0)

const mqtt = {
  //刷新mqtt状态
  refreshStatus() {
    _pre_.mqtt.refreshStatus()
  },

  /*
   * 发布一个本地的mqtt,只会通知本机的订阅者
   * @param {String} topic 主题
   * @param {mixni} message 内容
   * */
  localPublish(topic: string, message?: any) {
    _pre_.mqtt.localPublish(topic, message)
  },

  //订阅
  subscribes(subs: ff.mqtt.Subs) {
    let tops = Object.keys(subs) //主题数组
    _pre_.mqtt.subscribes(tops, webContentId)

    for (let i in subs) {
      if (topics[i]) {
        let topic = topics[i]

        //对比两个函数是否一样
        let isRepetition = topic.some(v => v === subs[i])

        //把执行函数加到对应的主题上
        if (!isRepetition) {
          topic.push(subs[i])
        }
      } else {
        //把执行函数加到对应的主题上
        topics[i] = [subs[i]]
      }
    }
  },

  //取消订阅
  unsubscribes(subs: ff.mqtt.Subs) {
    //存放需要取消订阅的主题
    //不是所有的主题都要取消订阅，有些主题可能存在多个function,
    //只有剩下一个function的时候，才是真正取消订阅
    let unSubList: string[] = []

    for (let i in subs) {
      if (topics[i]) {
        let topic = topics[i]

        if (topic.length > 1) {
          topic.some((v, index) => {
            if (v === subs[i]) {
              topic.splice(index, 1)
              return true
            }
          })
        } else if (topic.length === 1 && topic[0] === subs[i]) {
          unSubList.push(i)
          delete topics[i]
        }
      } else {
        unSubList.push(i)
      }
    }

    if (unSubList.length) {
      _pre_.mqtt.unsubscribes(unSubList, webContentId)
    }
  },

  publish(topic: string, message: any = {}) {
    let m: any
    if (typeof message === 'object') {
      message.$user_id = userInfo?.id ?? 0
      m = JSON.stringify(message)
    } else if (typeof message !== 'string') {
      m = String(message)
    } else {
      m = message
    }
    _pre_.mqtt.publish(topic, m)
    _pre_.mqtt.localPublish(topic, message)
  }
}

//自动向父组件注册该渲染层，使该渲染层能收到通知
_pre_.mqtt.onmessage(async (topic: string, message: any, isLocal?: boolean) => {
  if (isLocal) {
    console.log('%c本地mqtt: ' + topic, 'color:#fff;background-color:#444;padding: 0px 1px; border-radius: 3px', message)
  } else {
    console.log('%c在线mqtt: ' + topic, 'color:#fff;background-color:#319537;padding: 0px 1px; border-radius: 3px', message)
  }

  /*
   * 分拣消息,分配消息通知到指定的位置
   * @param {String} topic 主题
   * @param {mixin} message 消息
   * */

  //获取订阅主题的数量
  let funs: Function[] = []

  //直接配置的，没有通配符
  if (topics[topic] && topics[topic].length) {
    funs = funs.concat(topics[topic])
  }

  //通配符主题
  for (let i in topics) {
    if (i.indexOf('/+') !== -1) {
      let d = i.replace(/\+/g, '[^/]+').replace(/\$/g, '\\$')
      let reg = new RegExp('^' + d + '$')

      if (reg.test(topic)) {
        if (topics[i] && topics[i].length) {
          funs = funs.concat(topics[i])
        }
      }
    }
  }

  /************************************/
  if (!funs.length) {
    console.log('没有订阅该主题', topic)
    return
  }

  if (funs.length) {
    let send = function (index: number) {
      if (typeof funs[index] === 'function') {
        funs[index](message, topic)
        send(index + 1)
      }
    }

    send(0)
  }
})

window.addEventListener('unload', () => {
  _pre_.mqtt.removeClient(webContentId)
})

export default mqtt

==> ff-global-4.0.9.tgz: package/renderer/src/mqtt.ts <==
//在渲染器进程 (网页) 中。

/*
 * 键：表示已经订阅的主题
 * 值：是一个数组，执行函数
 * */

const topics: {[topic: string]: Function[]} = {}
//当前渲染层id
const webContentId: number = _pre_.getWebContentId()

let userId = 0

const mqtt = {
  //刷新mqtt状态
  refreshStatus() {
    _pre_.mqtt.refreshStatus()
  },

  /*
   * 发布一个本地的mqtt,只会通知本机的订阅者
   * @param {String} topic 主题
   * @param {mixni} message 内容
   * */
  localPublish(topic: string, message?: any) {
    _pre_.mqtt.localPublish(topic, message)
  },

  //订阅
  subscribes(subs: ff.mqtt.Subs) {
    let tops = Object.keys(subs) //主题数组
    _pre_.mqtt.subscribes(tops, webContentId)

    for (let i in subs) {
      if (topics[i]) {
        let topic = topics[i]

        //对比两个函数是否一样
        let isRepetition = topic.some(v => v === subs[i])

        //把执行函数加到对应的主题上
        if (!isRepetition) {
          topic.push(subs[i])
        }
      } else {
        //把执行函数加到对应的主题上
        topics[i] = [subs[i]]
      }
    }
  },

  //取消订阅
  unsubscribes(subs: ff.mqtt.Subs) {
    //存放需要取消订阅的主题
    //不是所有的主题都要取消订阅，有些主题可能存在多个function,
    //只有剩下一个function的时候，才是真正取消订阅
    let unSubList: string[] = []

    for (let i in subs) {
      if (topics[i]) {
        let topic = topics[i]

        if (topic.length > 1) {
          topic.some((v, index) => {
            if (v === subs[i]) {
              topic.splice(index, 1)
              return true
            }
          })
        } else if (topic.length === 1 && topic[0] === subs[i]) {
          unSubList.push(i)
          delete topics[i]
        }
      } else {
        unSubList.push(i)
      }
    }

    if (unSubList.length) {
      _pre_.mqtt.unsubscribes(unSubList, webContentId)
    }
  },

  publish(topic: string, message: any = {}) {
    let m: any
    if (typeof message === 'object') {
      message.$user_id = userId
      m = JSON.stringify(message)
    } else if (typeof message !== 'string') {
      m = String(message)
    } else {
      m = message
    }
    _pre_.mqtt.publish(topic, m)
    _pre_.mqtt.localPublish(topic, message)
  },
  //设置用户id，用于排除本地通知与线上通知重复
  setUserId(id:number){
    userId = id
  }
}

//自动向父组件注册该渲染层，使该渲染层能收到通知
_pre_.mqtt.onmessage(async (topic: string, message: any, isLocal?: boolean) => {
  if (isLocal) {
    console.log('%c本地mqtt: ' + topic, 'color:#fff;background-color:#444;padding: 0px 1px; border-radius: 3px', message)
  } else {
    console.log('%c在线mqtt: ' + topic, 'color:#fff;background-color:#319537;padding: 0px 1px; border-radius: 3px', message)
  }

  /*
   * 分拣消息,分配消息通知到指定的位置
   * @param {String} topic 主题
   * @param {mixin} message 消息
   * */

  //获取订阅主题的数量
  let funs: Function[] = []

  //直接配置的，没有通配符
  if (topics[topic] && topics[topic].length) {
    funs = funs.concat(topics[topic])
  }

  //通配符主题
  for (let i in topics) {
    if (i.indexOf('/+') !== -1) {
      let d = i.replace(/\+/g, '[^/]+').replace(/\$/g, '\\$')
      let reg = new RegExp('^' + d + '$')

      if (reg.test(topic)) {
        if (topics[i] && topics[i].length) {
          funs = funs.concat(topics[i])
        }
      }
    }
  }

  /************************************/
  if (!funs.length) {
    console.log('没有订阅该主题', topic)
    return
  }

  if (funs.length) {
    let send = function (index: number) {
      if (typeof funs[index] === 'function') {
        funs[index](message, topic)
        send(index + 1)
      }
    }

    send(0)
  }
})

window.addEventListener('unload', () => {
  _pre_.mqtt.removeClient(webContentId)
})

export default mqtt
